//
// Programmer:    Craig Stuart Sapp <craig@ccrma.stanford.edu>
// Creation Date: Mon Feb 21 19:22:56 PST 2000
// Last Modified: Mon Mar  6 14:21:16 PST 2000
// Filename:      BatonCanvas.cpp
// Syntax:        C++ 
// $Smake:        g++ -O3 -I/usr/lib/qt-2.1.0/include -c -g %f
//
// Description:   This class plots the baton data.
//

#include "BatonCanvas.h"
#include "BatonCanvas.moc"
#include <qpainter.h>
#include <qcolor.h>
#include <iostream.h>
#include <ctype.h>
#include <unistd.h>


//////////////////////////////
//
// BatonCanvas::BatonCanvas --
//     default values: parent=0, name=0
//

BatonCanvas::BatonCanvas(QWidget* parent, const char* name) :
      QWidget(parent, name) {
   pointsizex = 4;               // x size of point = 4 pixels by default
   pointsizey = 4;               // y size of point = 4 pixels by default

   printer = new QPrinter;

   plotStyle = BATON_PLOT_UNDEFINED;
   setPlotStyle(BATON_PLOT_SCROLL);
   setBackgroundColor(white);
}



//////////////////////////////
//
// BatonCanvas::~BatonCanvas --
//

BatonCanvas::~BatonCanvas() {
   delete printer;
}



//////////////////////////////
//
// BatonCanvas::clear --
//

void BatonCanvas::clear(void) {
   erase();
}



//////////////////////////////
//
// BatonCanvas::draw --
//     default values: y=0, z=0, t=0
//

void BatonCanvas::draw(int x, int y, int z, int t) {
   switch (getPlotStyle()) {
      case BATON_PLOT_XY:
         drawXYPlot(x, y);
         break;
      case BATON_PLOT_SCROLL:
         drawScrollPlot(z);
         break;
      case BATON_PLOT_WRAP:
         drawWrapPlot(z);
         break;
      default:
         break;
   }
}


//////////////////////////////
//
// BatonCanvas::drawScrollPlot --
//

void BatonCanvas::drawScrollPlot(int x) {
   int i;
   QBrush brush(Qt::black);
   QBrush brush2(Qt::red);           
   if (points2.getSize() == points2.getCount()) {
      points2.extract();
   }
   points2.insert(x);             // new point set

   painter.begin(this);

   if (points.getSize() == 0) {
      painter.fillRect(
         getCanvasSizeX() - getPointSizeX(), 
         getPointSizeY() * points2[0], 
         getPointSizeX(), getPointSizeY(), brush);
   } else if (points2[0] != points[0]) {
      painter.fillRect(
         getCanvasSizeX() - getPointSizeX(), 
         getPointSizeY() * points2[0], 
         getPointSizeX(), getPointSizeY(), brush);
   }

   for (i=0; i<points.getCount(); i++) {
      if (points2[i] == points[i]) {
         // do nothing
      } else {
         painter.eraseRect(
            getCanvasSizeX() - (i+1) * getPointSizeX(), 
            getPointSizeY() * points[i], 
            getPointSizeX(), getPointSizeY());

         painter.fillRect(
            getCanvasSizeX() - (i+1) * getPointSizeX(), 
            getPointSizeY() * points2[i], 
            getPointSizeX(), getPointSizeY(), brush);
      }
   }

   if (points.getCount() < points.getSize()) {
      painter.fillRect(
         getCanvasSizeX() - (points.getCount()+1) * getPointSizeX(), 
         getPointSizeY() * points2[points.getCount()],
         getPointSizeX(), getPointSizeY(), brush);
   } else if (points.getSize() == points.getCount()) {
      points.extract();
   }
   points.insert(x);              // old point set
   painter.end();
}



//////////////////////////////
//
// BatonCanvas::drawWrapPlot --
//

void BatonCanvas::drawWrapPlot(int x) {
   int i;
   QBrush brush(Qt::black);
   QBrush brush2(Qt::red);           
   if (points2.getSize() == points2.getCount()) {
      points2.extract();
   }
   points2.insert(x);             // new point set

   painter.begin(this);

   painter.eraseRect(
         eraseLoc * getPointSizeX(), 
         0,
         getPointSizeX(), getCanvasSizeY());

   painter.fillRect(
         writeLoc * getPointSizeX(), 
         x * getPointSizeY(),
         getPointSizeX(), getPointSizeY(), brush);
 
   eraseLoc++;
   writeLoc++;

   if (eraseLoc >= getCanvasSizeX()/getPointSizeX()) {
      eraseLoc = 0;
   }
   if (writeLoc >= getCanvasSizeX()/getPointSizeX()) {
      writeLoc = 0;
   }

   points.insert(x);              // old point set
   painter.end();
}



//////////////////////////////
//
// BatonCanvas::drawXYPlot --
//

void BatonCanvas::drawXYPlot(int x, int y) {
   QBrush brush(black);           
   int oldx = -1;
   int oldy = -1;

   if (xpoints.getSize() == xpoints.getCount()) {
      oldx = xpoints.extract();
   }
   if (ypoints.getSize() == ypoints.getCount()) {
      oldy = ypoints.extract();
   }
   xpoints.insert(x);
   ypoints.insert(y);

   painter.begin(this);

   if (present[x + 128*y] == 0) {
      painter.fillRect(
         getCanvasSizeX() - x * getPointSizeX(), 
         getCanvasSizeY() - y * getPointSizeY(),
         getPointSizeX(), getPointSizeY(), brush);
   }
   present[x + 128 * y]++;

   if (oldx != -1) {
      present[oldx + 128*oldy]--;
//      cout << "Position count: " << present[oldx + 128*oldy] << endl;
      if (present[oldx + 128*oldy] <= 0) {

      painter.eraseRect(
         getCanvasSizeX() - oldx * getPointSizeX(), 
         getCanvasSizeY() - oldy * getPointSizeY(),
         getPointSizeX(), getPointSizeY());
 
      }
   }

   painter.end();
}



//////////////////////////////
//
// BatonCanvas::getPlotStyle -- returns the current plotting style
//   on the canvas
//

int BatonCanvas::getPlotStyle(void) const {
   return plotStyle;
}



//////////////////////////////
//
// BatonCanvas::getCanvasSizeX -- returns the x dimension size of
//   the canvas
//

int BatonCanvas::getCanvasSizeX(void) const {
   return totalsizex;
}



//////////////////////////////
//
// BatonCanvas::getCanvasSizeY -- returns the y dimension size of
//   the canvas
//

int BatonCanvas::getCanvasSizeY(void) const {
   return totalsizey;
}



//////////////////////////////
//
// BatonCanvas::getPointSizeX -- returns the x dimension size of
//   a point
//

int BatonCanvas::getPointSizeX(void) const {
   return pointsizex;
}



//////////////////////////////
//
// BatonCanvas::getPointSizeY -- returns the x dimension size of
//   a point
//

int BatonCanvas::getPointSizeY(void) const {
   return pointsizey;
}


///////////////////////////////////////////////////////////////////////////
//
// Events
//


///////////////////////////////
//
// BatonCanvas::paintEvent -- Called when the widget needs to be updated.
//

void BatonCanvas::paintEvent(QPaintEvent *event) {
   QPainter paint(this);
}



///////////////////////////////
//
// BatonCanvas::keyPressEvent -- Called when the keyboard is hit.
//
// int checkKeyboard(int key = -1);
//

void BatonCanvas::keyPressEvent(QKeyEvent *k) {
//    checkKeyboard(k->ascii());
}



///////////////////////////////
//
// BatonCanvas::print -- Called when you want to print the current
//    contents of the display.
//

void BatonCanvas::print(void) {
cout << "Starting to print" << endl;
   if (printer->setup()) {
      QPainter qp(printer);
      switch (getPlotStyle()) {
         case BATON_PLOT_XY:
            printXYPlot(qp);
            break;
         case BATON_PLOT_SCROLL:
            printScrollPlot(qp);
            break;
         case BATON_PLOT_WRAP:
            printWrapPlot(qp);
            break;
         default:
            break;
      }
   }
cout << "Sleeping" << endl;
usleep(10000000);
cout << "Done printing " << endl;
}    



///////////////////////////////
//
// BatonCanvas::printXYPlot -- print the XY Plot.
//

void BatonCanvas::printXYPlot(QPainter& painter) {
   QBrush brush(black);
   for (int i=0; i<xpoints.getCount(); i++) {
      painter.fillRect(
         getCanvasSizeX() - xpoints[i] * getPointSizeX(),
         getCanvasSizeY() - ypoints[i] * getPointSizeY(),
         getPointSizeX(), getPointSizeY(), brush);
   }
}



///////////////////////////////
//
// BatonCanvas::printScrollPlot -- print the Scroll Plot.
//

void BatonCanvas::printScrollPlot(QPainter& painter) {

}



///////////////////////////////
//
// BatonCanvas::printWrapPlot -- print the Wrap Plot.
//

void BatonCanvas::printWrapPlot(QPainter& painter) {

}



///////////////////////////////
//
// BatonCanvas::sizeHint -- for use with QVBox
//

QSize BatonCanvas::sizeHint(void) {
   QSize output;
   switch (getPlotStyle()) {
      case BATON_PLOT_XY:
         output.setHeight(128 * getCanvasSizeX());
         output.setWidth(128 * getCanvasSizeY());
         break;
      case BATON_PLOT_SCROLL:
         output.setHeight(128 * getCanvasSizeX());
         output.setWidth(256 * getCanvasSizeY());
         break;
      case BATON_PLOT_WRAP:
         output.setHeight(128 * getCanvasSizeX());
         output.setWidth(256 * getCanvasSizeY());
         break;
      default:
         output.setHeight(0);
         output.setWidth(0);
         break;
   }

   return output;
}



///////////////////////////////
//
// BatonCanvas::resizeEvent -- Called when the widget has been resized.
//    Moves the button group to the upper right corner
//    of the widget.
//

void BatonCanvas::resizeEvent(QResizeEvent *event) {

}



///////////////////////////////
//
// BatonCanvas::setPlotStyle -- 
//

void BatonCanvas::setPlotStyle(int aStyle) {
   plotStyle = aStyle;
   switch (aStyle) {
      case BATON_PLOT_XY:
         totalsizex = pointsizex * 128;
         totalsizey = pointsizey * 128;

         // data for drawXYPlot:
         xpoints.setSize(50);
         xpoints.reset();
         ypoints.setSize(50);
         ypoints.reset();
         present.setSize(128*128); 
         present.zero();

         updateGeometry();

         break;
      case BATON_PLOT_SCROLL:
         totalsizex = pointsizex * 256;
         totalsizey = pointsizey * 128;
         updateGeometry();

         // data for drawScroll:
         points.setSize(totalsizex/pointsizex);
         points.reset();
         points2.setSize(totalsizex/pointsizex);
         points2.reset();

         break;
      case BATON_PLOT_WRAP:
         totalsizex = pointsizex * 256;
         totalsizey = pointsizey * 128;
         updateGeometry();

         // data for drawScroll:
         points.setSize(totalsizex/pointsizex);
         points.reset();
         points2.setSize(totalsizex/pointsizex);
         points2.reset();

         writeLoc = 0;
         eraseLoc = 1;

         break;
      default:
         break;
   }

   emit resize();
   erase();
}




