//
// Programmer:    Craig Stuart Sapp <craig@ccrma.stanford.edu>
// Creation Date: Mon Feb 21 19:22:56 PST 2000
// Last Modified: Mon Mar  6 14:17:53 PST 2000
// Filename:      BatonCanvas.h
// Syntax:        C++ 
// $Smake:        /usr/lib/qt-2.1.0/bin/moc %f > %b.moc
//
// Description:   This class plots the baton data.
//

#ifndef _BATONCANVAS_H_INCLUDED
#define _BATONCANVAS_H_INCLUDED

#include <qprinter.h>
#include <qwidget.h>
#include <qcolor.h>
#include <qpainter.h>
#include "CircularBuffer.h"
#include "Array.h"


// Plot Styles:
#define   BATON_PLOT_UNDEFINED -1
#define   BATON_PLOT_XY         0
#define   BATON_PLOT_SCROLL     1
#define   BATON_PLOT_WRAP       2


class BatonCanvas : public QWidget {
    Q_OBJECT

   public:
                    BatonCanvas            (QWidget *parent = 0, 
                                                const char* name = 0);
                   ~BatonCanvas            ();
      void          draw                   (int x, int y=0, int z=0, int t=0);
      int           getPlotStyle           (void) const;
      int           getCanvasSizeX         (void) const;
      int           getCanvasSizeY         (void) const;
      int           getPointSizeX          (void) const;
      int           getPointSizeY          (void) const;

      QSize         sizeHint               (void);

   public slots:
      void          setPlotStyle           (int aStyle);
      void          print                  (void);
      void          clear                  (void);

   protected:
      void          drawXYPlot             (int x, int y);
      void          drawScrollPlot         (int x);
      void          drawWrapPlot           (int x);

      void          printXYPlot            (QPainter& paint);
      void          printScrollPlot        (QPainter& paint);
      void          printWrapPlot          (QPainter& paint);


      // windows events:
      void          paintEvent            (QPaintEvent *event);
      void          resizeEvent           (QResizeEvent *event);
      void          keyPressEvent         (QKeyEvent *k);

   private:
      int           plotStyle;           // curret plotting style
      int           pointsizex;          // x size of point
      int           pointsizey;          // y size of point
      int           totalsizex;          // total x size
      int           totalsizey;          // total x size

      // data for drawScroll:
      CircularBuffer<int> points;        // points being displayed
      CircularBuffer<int> points2;       // new points being displayed

      // data for drawXYPlot:
      CircularBuffer<int> xpoints;       // points being displayed
      CircularBuffer<int> ypoints;       // points being displayed
      Array<int>          present;       // num of pts sitting on one location

      // data for drawWrapPlot:
      int                 writeLoc;      // writing a point location
      int                 eraseLoc;      // reading a point location


      QPainter      painter;
      QPrinter     *printer;             // for printing the display

   signals:
      void          resize               (void);

};


#endif /* _BATONCANVAS_H_INCLUDED */



