//
// Programmer:    Craig Stuart Sapp <craig@ccrma.stanford.edu>
// Creation Date: Sat Feb 26 14:24:50 PST 2000
// Last Modified: Mon Mar  6 13:50:14 PST 2000
// Filename:      BatonDisplay.cpp
// Syntax:        C++; qt 2.1.0
// $Smake:        g++ -O3 -I/usr/lib/qt-2.1.0/include -g -c %f
//
// Description:   This is the primary GUI interface to draw baton position data.
//

#include "BatonDisplay.h"
#include "BatonDisplay.moc"
#include <qtoolbar.h>
#include <qmainwindow.h>


// static variables:
int            BatonDisplay::argcc = 0;
char**         BatonDisplay::argvv = NULL;
QApplication   BatonDisplay::qapplication(argcc, argvv);
int            BatonDisplay::displayState = 0;
BatonWidget   *BatonDisplay::mainwindow = NULL;
BatonMenu     *BatonDisplay::menu = NULL;
BatonCanvas   *BatonDisplay::canvas = NULL;


//////////////////////////////
//
// BatonDisplay::BatonDisplay --
//

BatonDisplay::BatonDisplay(void) {
   mainwindow = new BatonWidget;
   menu = new BatonMenu(mainwindow);    
   canvas = new BatonCanvas(mainwindow);

/*
   // make the menu movable
   QToolBar *toolbar = new QToolBar(mainwindow);
   mainwindow->addToolBar(toolbar, tr("Menubar"), QMainWindow::Top, TRUE);
   menu = new BatonMenu(toolbar);
   menu->setFrameStyle( QFrame::NoFrame);
   toolbar->setStretchableWidget(menu);
   mainwindow->setDockEnabled(toolbar, QMainWindow::Left, FALSE);
   mainwindow->setDockEnabled(toolbar, QMainWindow::Right, FALSE);
*/

   mainwindow->setCentralWidget(canvas);

   setDisplayState(0);
   qapplication.setMainWidget(mainwindow);

   
//   mainwindow->resize(canvas->getCanvasSizeX(),
//         canvas->getCanvasSizeY() + menu->height());

   // mainwindow->show();


   // setup signal and slot connections for the display

   // when the menu item "Quit" is selected, call the
   // display interface's quit function (inherited from
   // the class BatonInterface).
   connect(menu, SIGNAL(quit()), this, SLOT(quit()));

   // when the menu item "Hide" is selected, call the
   // display interface's quit function (inherited from
   // the class BatonInterface).
   connect(menu, SIGNAL(hide()), this, SLOT(hide()));

   // when the menu item "XY" is selected, call the
   // canvas's intialize plot XY function:
   connect(menu, SIGNAL(plotStyle(int)), canvas, SLOT(setPlotStyle(int)));

   // when the canvas changes size, also change the mainwindow's size
   connect(canvas, SIGNAL(resize()), this, SLOT(resizeMain()));
   connect(menu, SIGNAL(resize()), this, SLOT(resizeMain()));

   // set the Print menu item to print the canvas
   connect(menu, SIGNAL(print()), canvas, SLOT(print()));

   // clear the plotting area when the menu "Clear" is chosen
   connect(menu, SIGNAL(clear()), canvas, SLOT(clear()));

   resizeMain();
}



//////////////////////////////
//
// BatonDisplay::~BatonDisplay --
//

BatonDisplay::~BatonDisplay() { 
   // nothing for now; Qt widgets are automatically deleted by
   // QApplication
}




//////////////////////////////
//
// BatonInterface::baton1pos -- input a new set of poll positions
//   from the Radio Batons for baton1
//     default values: yPoint and zPoint = 0, aTime = 0.
//

void BatonDisplay::baton1pos(int xPoint, int yPoint, int zPoint, int aTime) { 
   canvas->draw(xPoint, yPoint, zPoint, aTime);
}



//////////////////////////////
//
// BatonInterface::baton2pos -- input a new set of poll positions
//   from the Radio Batons for baton1
//     default values: yPoint and zPoint = 0, aTime = 0.
//

void BatonDisplay::baton2pos(int xPoint, int yPoint, int zPoint, int aTime) { 

}



//////////////////////////////
//
// BatonDisplay::exec -- run the application in an X-windows loop.
//

int BatonDisplay::exec(void) { 
   return qapplication.exec();
}



//////////////////////////////
//
// BatonDisplay::hide -- hide the canvas display.
//

void BatonDisplay::hide(void) { 
   if (viewMode() != 0) {
     mainwindow->hide(); 
     mainwindow->update(); 
     setDisplayState(0);
   }
}



//////////////////////////////
//
// BatonDisplay::processEvents -- handle all X-windows events such
//   as mouse clicks.
//

void BatonDisplay::processEvents(void) { 
   qapplication.processEvents();
}


   

//////////////////////////////
//
// BatonDisplay::resizeMain -- resize the main window
//

void BatonDisplay::resizeMain(void) { 
   mainwindow->resize(canvas->getCanvasSizeX(),
         canvas->getCanvasSizeY() + menu->height());
}


//////////////////////////////
//
// BatonDisplay::show -- display the canvas.
//

void BatonDisplay::show(void) { 
   if (viewMode() != 1) {
     mainwindow->show(); 
     mainwindow->update(); 
     setDisplayState(1);
   }
}



//////////////////////////////
//
// BatonDisplay::toggleView -- display or hide the canvas as necessary.
//

int BatonDisplay::toggleView(void) { 
   if (viewMode() == 0) {
      show();
   } else {
      hide();
   }

   return viewMode();
}



//////////////////////////////
//
// BatonDisplay::viewMode -- 0=hidden, 1=showing.
//

int BatonDisplay::viewMode(void) { 
   return displayState;
}




///////////////////////////////////////////////////////////////////////////
//
// private functions
//


//////////////////////////////
//
// BatonDisplay::setDisplayState -- 0=hidden, 1=showing.
//

void BatonDisplay::setDisplayState(int aState) {
   if (aState) {
      displayState = 1;
   } else {
      displayState = 0;
   }
}



