//
// Programmer:    Craig Stuart Sapp <craig@ccrma.stanford.edu>
// Creation Date: Mon Mar  6 14:35:12 PST 2000
// Last Modified: Mon Mar  6 14:35:17 PST 2000
// Filename:      BatonMenu.cpp
// Syntax:        C++; qt 2.1.0
// $Smake:        g++ -O3 -I/usr/lib/qt-2.1.0/include -g -c %f
//
// Description:   This is the menu for the radio baton plotting
//                display.
//

#include "BatonMenu.h"
#include "BatonDisplay.h"
#include "BatonInterface.h"
#include "BatonMenu.moc"
#include <qmessagebox.h>
#include <iostream.h>


//////////////////////////////
//
// BatonMenu::BatonMenu --
//     default values: parent = NULL; name = NULL;
//

BatonMenu::BatonMenu(QWidget* parent, const char* name) :
      QMenuBar(parent, name) {

    QPopupMenu *file = new QPopupMenu(this);
    CHECK_PTR(file);

    file->insertItem("&Print", this, SIGNAL(print()), CTRL+Key_P );        
    file->insertItem("Hide", this, SIGNAL(hide()), SHIFT+Key_V);
    file->insertItem("Q&uit",  this, SIGNAL(quit()), Key_Escape);


/*
    QPopupMenu *plot = new QPopupMenu(this);
    CHECK_PTR(plot);
    plot->insertTearOffHandle();
    plot->insertItem("&X-Y", this, SLOT(plotStyleXY()), 
          CTRL+Key_X);
    plot->insertItem("&Scroll", this, SLOT(plotStyleScroll()),
          CTRL+Key_S);
    plot->insertItem("&Wrap", this, SLOT(plotStyleWrap()), 
          CTRL+Key_W);
*/


    QPopupMenu *view = new QPopupMenu(this);
    CHECK_PTR(file);
    view->setCaption("View");
    view->insertTearOffHandle();
    view->insertItem("&X-Y", this, SLOT(plotStyleXY()), 
          CTRL+Key_X);
    view->insertItem("&Scroll", this, SLOT(plotStyleScroll()),
          CTRL+Key_S);
    view->insertItem("&Wrap", this, SLOT(plotStyleWrap()), 
          CTRL+Key_W);
    view->insertSeparator();
    view->insertItem("&Clear", this, SIGNAL(clear()), CTRL+Key_R);
    view->insertItem("&Pause", this, SLOT(pause()), CTRL+Key_U);
    view->insertSeparator();
    view->insertItem("&Options",  this, SLOT(pause()), CTRL+Key_O);

    QPopupMenu *help = new QPopupMenu(this);
    CHECK_PTR(help);
    help->insertItem("&About", this, SLOT(about()), CTRL+Key_H);

    this->insertItem("&File", file);
    this->insertItem("&View", view);
    this->insertSeparator();
    this->insertItem("&Help", help);
    this->setSeparator(QMenuBar::InWindowsStyle);

}



//////////////////////////////
//
// BatonMenu::~BatonMenu --
//

BatonMenu::~BatonMenu() {
   // do nothing
}



//////////////////////////////
//
// BatonMenu::about -- display
//

void BatonMenu::about(void) {
   QMessageBox::about(this, 
         "Radio Baton Display",
         "baton improv GUI Interface, 6 March 2000,\n"
         "written by Craig Stuart Sapp <craig@ccrma.stanford.edu>.\n"
   );
}



//////////////////////////////
//
// BatonMenu::pause -- 
//

void BatonMenu::pause(void) {
cout << "Going to pause the display" << endl;
}



//////////////////////////////
//
// BatonMenu::plotStyleXY -- 
//

void BatonMenu::plotStyleXY(void) {
   emit plotStyle(BATON_PLOT_XY);
}



//////////////////////////////
//
// BatonMenu::plotStyleScroll -- 
//

void BatonMenu::plotStyleScroll(void) {
   emit plotStyle(BATON_PLOT_SCROLL);
}



//////////////////////////////
//
// BatonMenu::plotStyleWrap -- 
//

void BatonMenu::plotStyleWrap(void) {
   emit plotStyle(BATON_PLOT_WRAP);
}



//////////////////////////////
//
// BatonMenu::reset -- 
//

void BatonMenu::reset(void) {
cout << "Going to reset the display" << endl;
}

